@@ -1,9 +1,12 @@
+from __future__ import annotations
+
+
 try:
-    import bpython  # noqa
+    import bpython
     # Access a property to verify module exists in case
     # there's a demand loader wrapping module imports
     # See https://github.com/inducer/pudb/issues/177
-    bpython.__version__
+    bpython.__version__  # noqa: B018
 except ImportError:
     HAVE_BPYTHON = False
 else:
@@ -66,17 +69,21 @@ custom_shell_dict = {}
 SHELL_FIRST_TIME = [True]
 
 
-def run_classic_shell(globals, locals):
+def run_classic_shell(globals, locals, message=""):
     if SHELL_FIRST_TIME:
         banner = "Hit Ctrl-D to return to PuDB."
         SHELL_FIRST_TIME.pop()
     else:
         banner = ""
 
+    if message:
+        banner = f"{message}\n{banner}"
+
     ns = SetPropagatingDict([locals, globals], locals)
 
-    from pudb.settings import get_save_config_path
     from os.path import join
+
+    from pudb.settings import get_save_config_path
     hist_file = join(
             get_save_config_path(),
             "shell-history")
@@ -127,7 +134,7 @@ def have_ipython():
         # Access a property to verify module exists in case
         # there's a demand loader wrapping module imports
         # See https://github.com/inducer/pudb/issues/177
-        IPython.core
+        IPython.core  # noqa: B018
     except (ImportError, ValueError):
         # Old IPythons versions (0.12?) may fail to import with
         # ValueError: fallback required, but not specified
@@ -196,8 +203,7 @@ def run_ipython_shell_v11(globals, locals):
         from IPython.terminal.interactiveshell import TerminalInteractiveShell
         from IPython.terminal.ipapp import load_default_config
     except ImportError:
-        from IPython.frontend.terminal.interactiveshell import \
-                TerminalInteractiveShell
+        from IPython.frontend.terminal.interactiveshell import TerminalInteractiveShell
         from IPython.frontend.terminal.ipapp import load_default_config
     # XXX: in the future it could be useful to load a 'pudb' config for the
     # user (if it exists) that could contain the user's macros and other
@@ -220,7 +226,17 @@ def run_ipython_shell_v11(globals, locals):
         args.append(banner)
     else:
         print(banner)
+
+    # XXX Quick and dirty way to fix issues with IPython 8.0.0+, introduced
+    # by commit 08d54c0e367b535fd88aca5273fd09e5e70d08f8.
+    # Setting _atexit_once_called = True will prevent call to
+    # IPython.core.interactiveshell.InteractiveShell._atexit_once() from inside
+    # IPython.terminal.interactiveshell.TerminalInteractiveShell.mainloop()
+    # This allows us to repeatedly re-call mainloop() and the whole
+    # run_ipython_shell_v11() function
+    shell._atexit_once_called = True
     shell.mainloop(*args)
+    del shell._atexit_once_called
 
     # Restore originating namespace
     _update_ipython_ns(shell, old_globals, old_locals)
@@ -249,8 +265,10 @@ def run_ipython_kernel(globals, locals):
 
 def get_ptpython_history_file():
     from argparse import ArgumentParser
+
     from ptpython.entry_points.run_ptpython import (  # pylint: disable=import-error
-            get_config_and_history_file)
+        get_config_and_history_file,
+    )
     parser = ArgumentParser()
     parser.add_argument("--history_file")
     parser.add_argument("--config_file")
